#include "Mouse.hpp"

#include <Graphics/Graphics.hpp>
#include <Graphics/Texture.hpp>
#include <Graphics/Font.hpp>
#include <Vector/VectorGL.hpp>

#include <utility>

using std::vector;
using std::pair;
using std::make_pair;

//A few constants to help us along:
namespace {
	const float Tick = 0.01f;
	const float Radius = 0.1f;
}

Mouse::Mouse() : mouse_pos(make_vector(0.0f, 0.0f)), active(true), time_acc(0.0f) {
	reset();
}


Mouse::~Mouse() {
}

void Mouse::reset() {
	blob.resize(200);
	for (unsigned int b = 0; b < blob.size(); ++b) {
		blob[b].pos = normalize(make_vector(rand() / float(RAND_MAX) - 0.5f, rand() / float(RAND_MAX) - 0.5f)) * (rand() / float(RAND_MAX));
		blob[b].prev = blob[b].pos;
	}
}

void Mouse::draw() {

	const unsigned int Angles = 100;
	vector< Vector2f > dirs(Angles);
	for (unsigned int a = 0; a < Angles; ++a) {
		float ang = a / float(Angles) * M_PI * 2.0f;
		dirs[a] = make_vector( cosf(ang), sinf(ang) );
	}

	vector< Vector2f > blob_quads(blob.size() * 4);
	vector< Vector2f > blob_texc(blob.size() * 4);
	for (unsigned int b = 0; b < blob.size(); ++b) {

		Vector2f right = dirs[b % dirs.size()];
		right *= Radius * 1.5f;
		Vector2f up = perpendicular(right);

		blob_texc[b*4+0] = make_vector(0.0f, 0.0f);
		blob_quads[b*4+0] = blob[b].pos - right - up;
		blob_texc[b*4+1] = make_vector(0.0f, 1.0f);
		blob_quads[b*4+1] = blob[b].pos - right + up;
		blob_texc[b*4+2] = make_vector(1.0f, 1.0f);
		blob_quads[b*4+2] = blob[b].pos + right + up;
		blob_texc[b*4+3] = make_vector(1.0f, 0.0f);
		blob_quads[b*4+3] = blob[b].pos + right - up;
	}

	glClearColor(0.9f, 0.9f, 0.9f, 0.0f);
	glClear(GL_COLOR_BUFFER_BIT);
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glVertexPointer(2, GL_FLOAT, sizeof(Vector2f), &(blob_quads[0]));
	glTexCoordPointer(2, GL_FLOAT, sizeof(Vector2f), &(blob_texc[0]));

	Graphics::TextureRef tex = Graphics::get_texture("textures/blob.png");
	if (tex.ref) {
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, tex->obj);
	}

	glColor4f(0.0f, 0.3f, 0.2f, 0.2f);
	glDrawArrays(GL_QUADS, 0, blob_quads.size());

	glDisable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, 0);

	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);

	glBegin(GL_TRIANGLES);
	glColor3f(0.7f, 0.2f, 0.1f);
	glVertex(mouse_pos);
	glVertex(mouse_pos + make_vector(0.03f,-0.04f));
	glVertex(mouse_pos + make_vector(0.0f,-0.05f));
	glEnd();

	Graphics::FontRef font = Graphics::get_font("gentium.txf");
	font->draw("move mouse, click-and-hold to stop controlling particle, ESCAPE -> menu", make_vector(-Graphics::aspectf(), -1.0f), 0.07f);
	font->draw_center("Mouse Demo", make_vector(0.0f, 0.9f), 0.1f);
}


class BlobEdge {
public:
	BlobEdge(unsigned int _a, unsigned int _b, float _len) : a(_a), b(_b), len(_len) {
	}
	unsigned int a,b;
	float len;
};

void Mouse::tick() {

	vector< BlobEdge > edges;
	for (unsigned int b1 = 0; b1 < blob.size(); ++b1) {
		for (unsigned int b2 = b1 + 1; b2 < blob.size(); ++b2) {
			if (length_squared(blob[b1].pos - blob[b2].pos) < Radius * Radius) {
				float len = length(blob[b1].pos - blob[b2].pos);
				if (len < 0.9f * Radius) {
					len += (0.9f * Radius - len) * 0.1f;
				}
				edges.push_back(BlobEdge(b1, b2, len));

			}
		}
	}

	for (unsigned int b = 0; b < blob.size(); ++b) {
		Vector2f old_pos = blob[b].pos;
		blob[b].pos = blob[b].pos * 2 - blob[b].prev;
		blob[b].pos += 0.5f * Tick * Tick * make_vector(0.0f, -1.0f);
		blob[b].prev = old_pos;
	}

	if (active) {
		blob[0].pos = mouse_pos;
		blob[0].prev = mouse_pos;
	}

	for (unsigned int iter = 0; iter < 2; ++iter) {
		for (unsigned int b = 0; b < blob.size(); ++b) {
			if (blob[b].pos.y + Radius > 1.0f) {
				blob[b].pos.y = 1.0f - Radius;
				blob[b].prev.y = blob[b].pos.y;
			}
			if (blob[b].pos.y - Radius <-1.0f) {
				blob[b].pos.y =-1.0f + Radius;
				blob[b].prev.y = blob[b].pos.y;
			}
			if (blob[b].pos.x + Radius > Graphics::aspectf()) {
				blob[b].pos.x = Graphics::aspectf() - Radius;
				blob[b].prev.x = blob[b].pos.x;
			}
			if (blob[b].pos.x - Radius <-Graphics::aspectf()) {
				blob[b].pos.x =-Graphics::aspectf() + Radius;
				blob[b].prev.x = blob[b].pos.x;
			}
		}
		for (vector< BlobEdge >::iterator e = edges.begin(); e != edges.end(); ++e) {
			BlobPt &a = blob[e->a];
			BlobPt &b = blob[e->b];
			Vector2f to = b.pos - a.pos;
			float len = length(to);
			if (len == 0.0f) continue;
			Vector2f push = 0.4f * to * (e->len - len) / len;
			a.pos -= push;
			b.pos += push;
		}
	}
}

void Mouse::update(float elapsed) {
	time_acc += elapsed;
	while (time_acc > 0.0f) {
		time_acc -= Tick;
		tick();
	}
}

void Mouse::handle_event(SDL_Event const &event) {
	if (event.type == SDL_MOUSEMOTION) {
		mouse_pos.x = event.motion.x / float(Graphics::screen.x) * 2.0f * Graphics::aspectf() - Graphics::aspectf();
		mouse_pos.y = event.motion.y / float(Graphics::screen.y) *-2.0f + 1.0f;
	}
	if (event.type == SDL_MOUSEBUTTONDOWN) {
		active = false;
	}
	if (event.type == SDL_MOUSEBUTTONUP) {
		active = true;
	}
	if (event.type == SDL_KEYDOWN) {
		if (event.key.keysym.sym == SDLK_ESCAPE) {
			quit_screen(this);
		}
	}
}
