#include "Textures.hpp"

#include <Graphics/Graphics.hpp>
#include <Graphics/Texture.hpp>
#include <Graphics/Font.hpp>
#include <Graphics/load_save_png.hpp>
#include <Basis/Log.hpp>
#include <Vector/QuatGL.hpp>

using std::vector;

Textures::Textures() : secret_mode(false) {
	rot.clear();
	rot_vel = make_vector(0.3f, 0.2614f, 0.387245f);

	//Load geometry stored as a "Geomtery Image" (as per the paper by Hoppe)
	vector< uint32_t > data;
	unsigned int w,h;
	if (!load_png("textures/geom.png", w, h, data)) {
		LOG_ERROR("Can't load texture for bunny in texture demo.");
		assert(0);
	}
	assert(data.size() == w*h);
	vector< uint32_t > data2;
	unsigned int w2,h2;
	if (!load_png("textures/norm.png", w2, h2, data2)) {
		LOG_ERROR("Can't load texture for bunny normals in texture demo.");
		assert(0);
	}
	assert(data2.size() == w2*h2);
	if (w != w2 || h != h2) {
		LOG_ERROR("geometry and normal images are different sizes.");
		assert(w == w2 && h == h2);
	}
	for (unsigned int y = 0; y + 1 < h; ++y) {
		for (unsigned int x = 0; x < w; ++x) {
			for (unsigned int ofs = 0; ofs < 2; ++ofs) {
				Vector3f val = make_vector< float >(*(Vector< uint8_t, 4 > *)(&data[(y + ofs) * w + x])).xyz;
				val *= 1.0f / 255.0f;
				val -= make_vector(0.5f, 0.5f, 0.5f);

				Vector3f norm = make_vector< float >(*(Vector< uint8_t, 4 > *)(&data2[(y + ofs) * w + x])).xyz;
				norm *= 2.0f / 255.0f;
				norm -= make_vector(1.0f, 1.0f, 1.0f);

				quad_strip.push_back(val);
				quad_strip_norms.push_back(norm);
				if ((x == 0 && ofs == 0) || (x + 1 == w && ofs == 1)) {
					quad_strip.push_back(val);
					quad_strip_norms.push_back(norm);
					quad_strip.push_back(val);
					quad_strip_norms.push_back(norm);
				}
			}
		}
	}
}

Textures::~Textures() {
}

void Textures::draw() {
	glClearColor(0.5f, 0.2f, 0.8f, 0.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	//Set up a perspective-projection camera to look at object:
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	gluPerspective(90.0f, Graphics::aspectf(),0.1f, 100.0f);

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();

	if (secret_mode) {
		gluLookAt(0.0f,0.0f,0.0f,  1.0f,0.0f,0.0f, 0.0f,0.0f,1.0f);
	} else {
		gluLookAt(1.0f,0.0f,0.0f,  0.0f,0.0f,0.0f, 0.0f,0.0f,1.0f);
	}


	//Rotate object:
	glRotate(rot);

	assert(quad_strip.size() == quad_strip_norms.size());

	Graphics::TextureRef tex = Graphics::get_texture("textures/ripple.png");

	//Draw object:
	glEnable(GL_DEPTH_TEST);
	if (!secret_mode) {
		glEnable(GL_LIGHTING);
		glEnable(GL_LIGHT0);
		glDisable(GL_LIGHT1); //sometimes enabled on osx
	}

	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_NORMAL_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);

	if (tex.ref) {
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, tex->obj);
	}

	glColor3f(1.0f, 1.0f, 1.0f);
	glVertexPointer(3, GL_FLOAT, sizeof(Vector3f), &quad_strip[0]);
	glNormalPointer(GL_FLOAT, sizeof(Vector3f), &quad_strip_norms[0]);
	//use x,y as texture coordinates. Same as using TexGen, but this
	//means you don't have to look up TexGen...
	glTexCoordPointer(2, GL_FLOAT, sizeof(Vector3f), &quad_strip[0]);
	glDrawArrays(GL_QUAD_STRIP, 0, quad_strip.size());
	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_NORMAL_ARRAY);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);


	glDisable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, 0);

	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);

	//remove perspective matrix
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

	Graphics::FontRef font = Graphics::get_font("gentium.txf");

	assert(font.ref);

	glColor3f(1.0f, 1.0f, 1.0f);
	font->draw_center("Texture Loading", make_vector(0.0f, 0.9f), 0.1f);
	glColor3f(0.7f, 0.7f, 0.7f);
	font->draw_center("ESCAPE to return to menu", make_vector(0.0f, 0.83f), 0.07f);
	glColor3f(1.0f, 1.0f, 1.0f);
	font->draw_center("(Yes, the bunny is supposed to be psychedelic and have a seam.)", make_vector(0.0f, 0.76f), 0.07f);

	Graphics::gl_errors("Textures::draw");
}

void Textures::update(float elapsed) {
	rot = multiply(rot, rotation(rot_vel.x*elapsed, make_vector(1.0f, 0.0f, 0.0f)));
	rot = multiply(rot, rotation(rot_vel.y*elapsed, make_vector(0.0f, 1.0f, 0.0f)));
	rot = multiply(rot, rotation(rot_vel.z*elapsed, make_vector(0.0f, 0.0f, 1.0f)));
	rot = normalize(rot);
}

void Textures::handle_event(SDL_Event const &event) {
	if (event.type == SDL_KEYDOWN) {
		if (event.key.keysym.sym == SDLK_ESCAPE) {
			quit_screen(this);
		}
		if (event.key.keysym.sym == SDLK_s) {
			//Good for you for reading the source.
			secret_mode = !secret_mode;
		}
	}
}
