#include "Timing.hpp"

#include <Graphics/Graphics.hpp>
#include <Graphics/Font.hpp>

#include <string>
#include <vector>

using std::string;
using std::vector;

const float Radius = 0.1f;

void Ball::update(float elapsed) {
	vel += elapsed * make_vector(0.0f, -0.1f); //Add Gravity
	//Fix velocity for collisions:
	if (pos.x - Radius < -Graphics::aspectf()) {
		if (vel.x < 0.0f) {
			vel.x *= -0.75f;
		}
	}
	if (pos.x + Radius > Graphics::aspectf()) {
		if (vel.x > 0.0f) {
			vel.x *= -0.75f;
		}
	}
	if (pos.y - Radius < -1.0f) {
		if (vel.y < 0.0f) {
			vel.y *= -0.75f;
		}
	}
	//Update position:
	pos += elapsed * vel;

}

Timing::Timing() : jitter(false) {
	reset_balls();
}

Timing::~Timing() {
}

void Timing::draw() {
	glClearColor(1.0f, 1.0f, 1.0f, 0.0f);
	glClear(GL_COLOR_BUFFER_BIT);


	//---------------------------------------
	//Draw a background grid:
	glBegin(GL_LINES);
	for (unsigned int y = 0; y < 15; ++y) {
		glColor4f(0.0f, 0.0f, 0.0f, 1.0f - (y / float(15.0f)));
		glVertex2f(-Graphics::aspectf(), y * 0.1f - 0.95f);
		glVertex2f( Graphics::aspectf(), y * 0.1f - 0.95f);
	}
	for (unsigned int x = 0; x * 0.1f < 2.0f * Graphics::aspectf(); ++x) {
		glColor4f(0.0f, 0.0f, 0.0f, 1.0f);
		glVertex2f(x * 0.1f - Graphics::aspectf(),-1.0f);
		glColor4f(0.0f, 0.0f, 0.0f, 0.0f);
		glVertex2f(x * 0.1f - Graphics::aspectf(), 0.5f);
	}
	glEnd();




	//---------------------------------------
	//Draw balls as circles:
	
	static vector< Vector2f > circle;
	if (circle.empty()) {
		circle.resize(93);
		for (unsigned int a = 0; a < circle.size(); ++a) {
			float ang = float(a) / float(circle.size()) * 2.0f * M_PI;
			circle[a] = make_vector(cosf(ang), sinf(ang));
		}
	}

	glEnableClientState(GL_VERTEX_ARRAY);
	glVertexPointer(2, GL_FLOAT, sizeof(Vector2f), &(circle[0]));

	for (unsigned int ball = 0; ball < 3; ++ball) {
		glPushMatrix();
		Vector3f col = make_vector(0.0f, 0.0f, 0.0f);
		if (ball == 0) {
			glTranslatef(per_frame.pos.x, per_frame.pos.y, 0.0f);
			col = make_vector(1.0f, 0.0f, 0.0f);
		} else if (ball == 1) {
			glTranslatef(fixed_step.pos.x, fixed_step.pos.y, 0.0f);
			col = make_vector(0.0f, 1.0f, 0.0f);
		} else if (ball == 2) {
			glTranslatef(variable_step.pos.x, variable_step.pos.y, 0.0f);
			col = make_vector(0.0f, 0.0f, 1.0f);
		}
		glScalef(Radius, Radius, 1.0f);
		glColor4f(col.r, col.g, col.b, 0.5f);
		glDrawArrays(GL_TRIANGLE_FAN, 0, circle.size());
		glColor3fv(col.c);
		glDrawArrays(GL_LINE_LOOP, 0, circle.size());
		glPopMatrix();
	}

	glDisableClientState(GL_VERTEX_ARRAY);

	//----------------------
	//Draw some help text:

	Graphics::FontRef font = Graphics::get_font("gentium.txf");
	assert(font.ref);

	glColor3f(0.0f, 0.0f, 0.0f);
	font->draw_center("Timing example.", make_vector(0.0f, 0.87f), 0.13f);
	glColor3f(0.3f, 0.3f, 0.3f);
	font->draw_center(string("SPACE to reset; 'j' to add frame length jitter (currently ") + (jitter?"on":"off") + string("); ESCAPE to return to menu."), make_vector(0.0f, 0.8f), 0.07f);
	{
		vector< string > names;
		names.push_back("red");
		names.push_back("green");
		names.push_back("blue");
		vector< string > methods;
		methods.push_back("updates every frame; it doesn't move smoothly if frame lengths vary");
		methods.push_back("updates with a fixed step");
		methods.push_back("updates with a variable step; it drifts away from the fixed-step over time");
		const float Height = 0.07f;
		float pos = 0.75f;
		for (unsigned int ball = 0; ball < 3; ++ball) {
			pos -= Height;
			float ofs = -0.5f * font->length("The " + names[ball] + " ball " + methods[ball], Height);
			glColor3f(0.0f, 0.0f, 0.0f);
			font->draw("The ", make_vector(ofs, pos), Height);
			ofs += font->length("The ", Height);
			glColor3f(ball == 0, ball == 1, ball == 2);
			font->draw(names[ball], make_vector(ofs, pos), Height);
			ofs += font->length(names[ball], Height);
			glColor3f(0.0f, 0.0f, 0.0f);
			font->draw(" ball " + methods[ball], make_vector(ofs, pos), Height);
		}
	}

	Graphics::gl_errors("Timing::draw");
}


void Timing::update(float elapsed) {
	{ //The per_frame ball assumes we are running at the current frame-rate cap:
		per_frame.update(min_ms_per_frame / 1000.0f);
	}
	{ //The fixed_step ball actually keeps track of how fast we are running:
		const float Tick = 0.01f;
		time_acc += elapsed;
		while (time_acc > 0.0f) {
			time_acc -= Tick;
			fixed_step.update(Tick);
		}
	}
	{ //The variable_step ball just uses the time elapsed since last frame:
		variable_step.update(elapsed);
	}

	if (jitter) {
		//simulate a variable-length computation:
		if (rand() % 2) {
			SDL_Delay(rand() % 50);
		}
	}
}

void Timing::handle_event(SDL_Event const &event) {
	if (event.type == SDL_KEYDOWN) {
		if (event.key.keysym.sym == SDLK_j) {
			jitter = !jitter;
		}
		if (event.key.keysym.sym == SDLK_SPACE) {
			reset_balls();
		}
		if (event.key.keysym.sym == SDLK_ESCAPE) {
			quit_screen(this);
			return;
		}
	}
}

void Timing::reset_balls() {
	per_frame.pos = make_vector(-0.5f,0.0f);
	per_frame.vel = make_vector( 0.3f,0.0f);

	fixed_step.pos = make_vector(-0.5f,0.0f);
	fixed_step.vel = make_vector( 0.3f,0.0f);
	time_acc = 0.0f;

	variable_step.pos = make_vector(-0.5f,0.0f);
	variable_step.vel = make_vector( 0.3f,0.0f);

}
